from .base import UploadBaseError


class ConnectError(UploadBaseError):
    code = 0x1001
    message = "connect error"


class LoginError(UploadBaseError):
    code = 0x1002
    message = "authentication error"


class SendDataError(UploadBaseError):
    code = 0x1003
    message = "send data error"


class ConfigurationError(UploadBaseError):
    code = 0x1004
    message = "server configuration error"



import os
from enum import Enum
from typing import Any, Optional

from pydantic import BaseModel


class FTPFileType(str, Enum):
    CSV = "csv"


class FTPData(BaseModel):
    file_type: FTPFileType
    file_name: str
    data: Any

    def build_full_path(self, remote_path) -> str:
        return os.path.join(remote_path, self.file_name)


class FTPServiceConfig(BaseModel):
    host: str
    username: Optional[str]
    password: Optional[str]
    remote_path: str


class FTPParsedData(BaseModel):
    file_name: str
    data: Any



import csv
import tempfile

from solax_py_library.upload.core.data_adapter.base import BaseDataAdapter


class CSVDataAdapter(BaseDataAdapter):
    @classmethod
    def parse_data(cls, data):
        with tempfile.NamedTemporaryFile(
            delete=False, mode="w", newline="", encoding="utf8"
        ) as temp_file:
            writer = csv.writer(temp_file)
            writer.writerows(data)

            temp_file_path = temp_file.name
            temp_file.seek(0)

        return open(temp_file_path, "rb")



from enum import Enum
from pydantic import BaseModel
from typing import Optional

from solax_py_library.upload.types.ftp import FTPData


class UploadType(str, Enum):
    FTP = "FTP"


class UploadData(BaseModel):
    ftp_data: Optional[FTPData]



import unittest

from solax_py_library.upload.api.service import upload
from solax_py_library.upload.types.client import UploadType, UploadData
from solax_py_library.upload.types.ftp import FTPData, FTPFileType


class FTPTest(unittest.TestCase):
    def test_ftp_upload(self):
        ftp_config = {
            "host": "10.1.31.181",  # 测试host
            "username": "solax",
            "password": "sly402",
            "remote_path": "/xixi",
        }
        upload(
            upload_type=UploadType.FTP,
            configuration=ftp_config,
            upload_data=UploadData(
                ftp_data=FTPData(
                    file_type=FTPFileType.CSV,
                    file_name="new_file.csv",
                    data=[["curry", 37, "男"], ["james", 40, "男"], ["lily", 16, "女"]],
                ),
            ),
        )



import ftplib
import os

from .base import BaseUploadService
from solax_py_library.upload.types.client import UploadType, UploadData
from solax_py_library.upload.core.data_adapter import CSVDataAdapter
from solax_py_library.upload.types.ftp import FTPServiceConfig, FTPFileType, FTPData, FTPParsedData
from solax_py_library.upload.errors.upload_error import SendDataError, ConnectError, ConfigurationError


class FTPUploadService(BaseUploadService):
    upload_type = UploadType.FTP

    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        config = self._init_config(**kwargs)
        self.host = config.host
        self.username = config.username
        self.password = config.password
        self.remote_path = config.remote_path

    def _init_config(self, **kwargs):
        try:
            return FTPServiceConfig(**kwargs)
        except Exception as e:
            print(e)
            raise ConfigurationError

    def connect(self):
        if self._client is not None:
            self.close()
        try:
            self._client = ftplib.FTP(self.host)
            self._client.login(self.username, self.password)
            print(f"Connected to {self.host}")
            self._is_connect = True
        except Exception as e:
            print(e)
            raise ConnectError

    def close(self):
        if self._client:
            self._client.quit()
            self._client = None
            self._is_connect = False
            print("Connection closed.")

    def _parse(self, upload_data: UploadData) -> FTPParsedData:
        data: FTPData = upload_data.ftp_data
        if data.file_type == FTPFileType.CSV:
            parsed_data = CSVDataAdapter.parse_data(data.data)
        else:
            raise NotImplementedError
        return FTPParsedData(
            file_name=data.build_full_path(self.remote_path), data=parsed_data
        )

    def _upload(self, data: FTPParsedData):
        try:
            if not self._is_connect:
                raise ConnectError(message="not connect yet")
            self._client.storbinary(
                f"STOR {self._build_remote_file(data.file_name)}", data.data
            )
            print(f"Successfully uploaded to {data.file_name}")
        except Exception as e:
            print(e)
            raise SendDataError

    def _build_remote_file(self, file_name):
        return os.path.join(self.remote_path, file_name)
